/*************************************************************
Project :  Test des fonctionnalits de la carte AT90S8535 SSI
Create :  8/03/2003
Version : 1
Rvision : 5     Date    : 03/03/2008
Author  : Philippe Mariano                
Company : LYCEE Pierre Emile MARTIN       
Comments: Test des BP, des DELs, des entres TOR, du Buzzer,
de l'entre analogique, du capteur de temprature DS1621,
de l'EEPROM externe, de l'HTR PCF8583, de la RS232, de la PWM
Chip type           : AT90S8535
Clock frequency     : 4,000000 MHz
Memory model        : Small
Internal SRAM size  : 512
External SRAM size  : 0
Data Stack size     : 152 !!!!!!!!!!!!!!!!!
************************************************************/

#include <90s8535.h> 

// Librairies non gnres par Wizard (rajoutes manuellement)
//-----------------------------------------------------------------      
#include <ssi.h>      // contient les fonctions Lire_BP,Affiche_LCD etc..
#include <delay.h>    // contient la fonction delay_ms
#include <stdio.h>    // contient la fonction sprintf 
#include <stdlib.h>   // contient la fonction itoa
#include <math.h>

// Accs aux caractres en CGRAM 
// --------------------------------
#define Caractere_FlecheH	0 
#define Caractere_FlecheB	1

// @ I2C HTR
// --------------------------------
#define	adresse_HTR	0

// Adresse CGRAM de l'afficheur LCD
// --------------------------------
#define Add_FlecheH_CGRAM 0x40 // @1 Caractre flche haut
#define Add_FlecheB_CGRAM 0x48 // @2 Caractre flche bas

// I2C Bus functions
#asm
   .equ __i2c_port=0x12
   .equ __sda_bit=3
   .equ __scl_bit=7
#endasm
#include <i2c.h>

// DS1621 Thermometer/Thermostat functions
#include <ds1621.h>

// PCF8583 Real Time Clock functions
#include <pcf8583.h>

// Alphanumeric LCD Module functions
#asm
   .equ __lcd_port=0x15
#endasm
#include <lcd.h>

// Read the AD conversion result
unsigned int read_adc(unsigned char adc_input)
{
ADMUX=adc_input;
// Start the AD conversion
ADCSR|=0x40;
// Wait for the AD conversion to complete
while ((ADCSR & 0x10)==0);
ADCSR|=0x10;
return ADCW;
}    


// Declare your global variables here
/*-----------------------------------------------------------------------------------------
type 		nom 				       		dsignation       
------------------------------------------------------------------------------------------*/
char display_buffer_ligne0[17]; 			// tampon ligne 0 de l'afficheur
char display_buffer_ligne1[17]; 	       		// tampon ligne 1 de l'afficheur
unsigned char heures, minutes, secondes, secondes_1, dizieme; 	//stockage de l'heure
unsigned char Heure_ou_Date, H_J, mn_mois;
unsigned int sec_annee;

enum Etat_Test {LedBP,LedTOR,Analog,Temperature,
 	Reglage_HTR,Horloge,Buzzer,EEPROM,RS232Tx,MOT_PWM,REG_PWM} etat=LedBP, etat_1=LedTOR;// Etats du graphe tests	 
 
// Constantes en flash
const char menu[]	= "  [ENTR]=>Oui   ";      
const char menuBP[]	= "Test? BP->DEL   "; 
const char menuTOR[] 	= "Test? TOR->DEL  ";
const char menuAnalog[] = "Test? E_Analog  "; 
const char menuBuzzer[] = "Test? Buzzer    "; 
const char menuEEPROM[] = "Test? EEPROM    ";  
const char menuRS232[]  = "Test? RS232Tx   "; 
const char MENU_PWM[]   = "Test? PWMA      ";

// Caractres personnaliss placs dans la mmoire CGRAM du LCD  
const unsigned char flecheHaut[8] = {0x04, 0x0E, 0x15, 0x04, 0x04, 0x04, 0x04, 0x00};    // @1 CGRAM
const unsigned char flecheBas[8] = {0x04, 0x04, 0x04, 0x04, 0x15, 0x0E, 0x04, 0x00};     // @2 CGRAM       


// Prototypes des fonctions
int Led_BP_TOR(char entree);  // test des Leds et des BPs 
void Affiche_fleches(void);

void main(void)
{ 

// Declare your local variables here
// -----------------------------------------------------------------------------------------
//type 		nom 					dsignation  
// -------------------------------------------------------------------------------------- --

char display_buffer[32]; 		// Tampon utilis lors de la lecture de la EEPROM
			 		// en page de 32 octets
unsigned char i, j, BP;
unsigned char jour,mois;
unsigned int  temp_n,annee;  
unsigned int  mesure,analogU, analogD, analogC; // Calcul de la ddp dlivre par le pot
unsigned char *Transm; 
unsigned char rcycl=0, rcycl_1=0;
unsigned char alpha[11] = {0,25,51,76,102,127,153,178,204,229,255};   // Coefficient pour le rglage du rapport cyclique du 
									// signal PWM
// Input/Output Ports initialization
// Port A
PORTA=0x3F; // Port A en entre avec rsistance de rappel
DDRA=0x00;

// Port B
PORTB=0x00; // Port B en sortie
DDRB=0x1F;

// Port C
PORTC=0x00; // Port C en sortie
DDRC=0xFF;  // Partag entre le LCD et le Buzzer(PORTC.3)

// Port D initialization
// Func0=In Func1=In Func2=In Func3=In Func4=In Func5=Out Func6=In Func7=In
// State0=T State1=T State2=T State3=T State4=T State5=0 State6=T State7=T
PORTD=0x00;
DDRD=0x20;

// Timer/Counter 1 initialization
// Clock source: System Clock
// Clock value: 4000,000 kHz
// Mode: Ph. correct PWM top=00FFh
// OC1A output: Non-Inv.
// OC1B output: Discon.
// Noise Canceler: Off
// Input Capture on Falling Edge
TCCR1A=0x81;
TCCR1B=0x01;
TCNT1H=0x00;
TCNT1L=0x00;
OCR1AH=0x00;
OCR1AL=0x00;
OCR1BH=0x00;
OCR1BL=0x00;

// ADC initialization
// ADC Clock frequency: 31,250 kHz
ADCSR=0x87;

// UART initialization
// Communication Parameters: 8 Data, 1 Stop, No Parity
// UART Receiver: Off
// UART Transmitter: On
// UART Baud rate: 19200
UCR=0x08;
UBRR=0x0C;

// I2C Bus initialization
i2c_init();

// DS1621 Thermometer/Thermostat initialization
// tlow: 24C
// thigh: 55C
// Tout polarity: 0
//ds1621_init(0,24,55,0);

// PCF8583 Real Time Clock initialization
rtc_init(0,0);

// LCD module initialization
lcd_init(16); 

// Chargement initial de la mmoire CGRAM de l'afficheur avec les 
// caractres personaliss
for (i=0;i<8;i++)
{
lcd_write_byte(Add_FlecheH_CGRAM + i, flecheHaut[i]); // @1 Caractre flche haut
lcd_write_byte(Add_FlecheB_CGRAM + i, flecheBas[i]); // @2 Caractre flche bas
}

sprintf(display_buffer_ligne0,"Test de la carte");
sprintf(display_buffer_ligne1,"******S.SI******");
Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
delay_ms(3000);

while (1)
      {		   // Dbut While(1)  
       
      BP = Lire_BP();
      switch (etat)
       {               // Dbut switch (etat)
        case LedBP :   if (etat_1 != etat)
                        { 
                         sprintf(display_buffer_ligne0,menuBP);
                         sprintf(display_buffer_ligne1,menu);
                         Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1); 
                         Affiche_fleches();
                        } 
                        switch(BP)
        		    {
        		     case INC : etat = LedTOR; break;
        		     case DEC : etat = MOT_PWM; break;
        		     case ENTR: Led_BP_TOR(0); //test des BP
       		      			sprintf(display_buffer_ligne0,menuBP);
                  			sprintf(display_buffer_ligne1,menu);
                     			Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1); 
                             break;
        		     default  : etat = LedBP;
        		    }
       			etat_1 = LedBP;
       		        
        break;
        
        case LedTOR : if (etat_1 != etat)
                        { 
                         sprintf(display_buffer_ligne0,menuTOR);
                         sprintf(display_buffer_ligne1,menu);
                         Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
                         Affiche_fleches();
                        } 
                        switch(BP)
        		    {
        		     case INC : etat = Buzzer; break;
        		     case DEC : etat = LedBP; break;
        		     case ENTR: Led_BP_TOR(1); // test des entres TOR
       		      			sprintf(display_buffer_ligne0,menuTOR);
                  			sprintf(display_buffer_ligne1,menu);
                     			Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1); 
                             break;
        		     default  : etat = LedTOR;
        		    }
       			etat_1 = LedTOR;	
        break;
        
        case Buzzer:	if (etat_1 != etat)
                        { 
                         sprintf(display_buffer_ligne0,menuBuzzer);
                         sprintf(display_buffer_ligne1,menu);
                         Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1); 
                         Affiche_fleches();
                        } 
        		switch(BP)
        		    {
        		     case INC : etat = Analog; break;
        		     case DEC : etat = LedTOR; break;
        		     default  : etat = Buzzer;
        		    }
       			PORTC.3 = !PINA.0;
       			etat_1 = Buzzer;       		
        break;  
        
        case Analog :  	if (etat_1 != etat)
                        { 
                         sprintf(display_buffer_ligne0,menuAnalog);
                         sprintf(display_buffer_ligne1,menu);
                         Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1); 
                         Affiche_fleches();
                        } 
                        switch(BP)
                         {
        		     case INC : etat = Temperature; break;
        		     case DEC : etat = Buzzer; break;
        		     case ENTR: sprintf(display_buffer_ligne1,"[ECHP]=>Quit    ");
       		       			          
       		      			while (BP != ECHAP) 
       		       			{ 
       		       			 BP = Lire_BP();
       		       			 delay_ms(500);
       		       			 mesure = (read_adc(6)/4); // Mesure et calcul de la ddp
       		       			 analogU = ((mesure*5)/255);
       		       			 analogD = ((10*((mesure - (analogU*255)/5))*5)/255);
       		       			 analogC = ((100*(mesure - (analogU*255)/5 -(analogD*255)/50)*5)/255);
       		       			 sprintf(display_buffer_ligne0,"  DDP = %-u.%-u%-uV",analogU, analogD, analogC);
       		       			 Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1); 
       		       			}
       		       			 sprintf(display_buffer_ligne0,menuAnalog);
                    			 sprintf(display_buffer_ligne1,menu);
                        		 Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);  
       		             break;
       			     default  : etat = Analog;
       			   }
       			etat_1 = Analog;       		
        break;
        
        case Temperature : temp_n = ds1621_temperature_10(0); // Affichage de la temprature toutes les secondes
        	           rtc_get_time(adresse_HTR,&heures,&minutes,&secondes,&dizieme);
        	       	   if ((etat_1 != etat) || (secondes_1 != secondes))
        		    {  
        		      sprintf(display_buffer_ligne0,"Capteur DS1621  ");
        		      sprintf(display_buffer_ligne1,"Temp. = %-i.%-u%-cC",temp_n/10,abs(temp_n%10),0xdf);
        		      Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
        	              secondes_1 = secondes;   
        	              Affiche_fleches();
        	            }
        	           switch(BP)
        		    {
        		     case INC : etat = EEPROM; break;
        		     case DEC : etat = Analog; break;
        		     default  : etat = Temperature;
        		    }
        		   etat_1 = Temperature;
        break;
        
        case EEPROM:	if (etat_1 != etat)
                        { 
                         sprintf(display_buffer_ligne0,menuEEPROM);
                         sprintf(display_buffer_ligne1,menu);
                         Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
                         Affiche_fleches(); 
                        }  
                        switch(BP)
        		    {
        		     case INC : etat = Horloge; break;
        		     case DEC : etat = Temperature; break;
        		     case ENTR: j=0;
       		      		        for (i=0;i<8;i++)
					 { 
 					   lcd_clear();
 					   eeprom_page_read(0xae,j,display_buffer); // EEPROM  l'@AE
 					   lcd_puts(display_buffer);
 					   delay_ms(4000);
 					   j=j+32;
					 }
        		     default  : etat = EEPROM;
        		    }
       			etat_1 = EEPROM;       		
        break;            
        
        case Horloge :  rtc_get_time(adresse_HTR,&heures,&minutes,&secondes,&dizieme);    
        		if ((etat_1 != etat) || (secondes_1 != secondes))
        		{
        		 rtc_get_date(adresse_HTR,&jour,&mois,&annee);
        		 sprintf(display_buffer_ligne0," %-u / %-u / %-i",jour,mois,annee);
        		 sprintf(display_buffer_ligne1,"%-uh / %-umn / %-us ",heures, minutes, secondes);
        		 Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
        	         secondes_1 = secondes;
        	         Affiche_fleches();
        	        }
        		switch(BP)
        		{
        		 case INC : etat = RS232Tx; break;
        		 case DEC : etat = EEPROM; break;
        		 case ENTR : etat = Reglage_HTR; break; 
        		 default : etat = Horloge;
        		}
        		etat_1 = Horloge;
        break;     
        
        case Reglage_HTR : if (etat_1 != etat)
                           {
        	            sprintf(display_buffer_ligne0,"  Reglage HTR");
        	            sprintf(display_buffer_ligne1,"SET>H ETR>Date ");
        	            Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
        		   }   
        		               
        	          switch(BP)
        		   {
        		    case SET   : Heure_ou_Date = 0;
        		  	        H_J = 12;
        		  	        mn_mois = 30;
        		  	        sec_annee = 0;
        		  	        regle_HTR(adresse_HTR,Heure_ou_Date,H_J,mn_mois,sec_annee);
        		  	        etat = Horloge;
        		    break;
        		 
        		    case ENTR : Heure_ou_Date = 1;
        		  	         H_J = 15;
        		  	         mn_mois = 6;
        		  	         sec_annee = 2008;
        		  	         regle_HTR(adresse_HTR,Heure_ou_Date,H_J,mn_mois,sec_annee);
        		  	         etat = Horloge;
        		    break;
        		 
        		    default:     etat = Reglage_HTR;
        		   } 
        	          etat_1 = Reglage_HTR;
        break; 
        
        case RS232Tx:	if (etat_1 != etat)
                        { 
                         sprintf(display_buffer_ligne0,menuRS232);
                         sprintf(display_buffer_ligne1,menu);
                         Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
                         Affiche_fleches(); 
                        }  
                        switch(BP)
        		    {
        		     case INC : etat = MOT_PWM; break;
        		     case DEC : etat = Horloge; break;
        		     case ENTR: rtc_get_time(adresse_HTR,&heures,&minutes,&secondes,&dizieme);
        		     sprintf(Transm,"%u/%u/%u",heures, minutes, secondes);
        		     puts(Transm); 
        		     break;       
        		     default  : etat = RS232Tx;
        		    }
       			etat_1 = RS232Tx;       		
        break; 
        
        case MOT_PWM :	if (etat_1 != etat)
                        { 
                         sprintf(display_buffer_ligne0,MENU_PWM);
                         sprintf(display_buffer_ligne1,menu);
                         Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
                         Affiche_fleches(); 
                        }  
                        switch(BP)
        		    {
        		     case INC : etat = LedBP; break;
        		     case DEC : etat = RS232Tx; break;
        		     case ENTR: etat = REG_PWM; break; 
        		     default  : etat = MOT_PWM;
        		    }
       			etat_1 = MOT_PWM;       
        break;
        
        case REG_PWM :  if ((etat_1 != etat)|| (rcycl_1 != rcycl))
       			  {
       		   	    sprintf(display_buffer_ligne0,"Rap. cycl = %-u%%",rcycl);
       		            sprintf(display_buffer_ligne1,"INC>+ DEC>- ESc>");
       		            Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1); 
       		            rcycl_1 = rcycl; 
			  }
		        switch(BP)
			      {
				case INC: if (rcycl < 90) rcycl = rcycl+10; else rcycl = 100; break;
				case DEC: if (rcycl > 10) rcycl = rcycl-10; else rcycl = 0; break;
				case ECHAP : rcycl = 0; etat = MOT_PWM; break;
				default: etat = REG_PWM;
                              }
			OCR1A = alpha[rcycl/10];
         		etat_1 = REG_PWM ;      		      
        break;
                   
        
       }; // Fin switch(etat)
      }  // Fin while(1)    
} // Fin main()

//===================================================================
// Sous-programmes
//-------------------------------------------------------------------
void Affiche_fleches(void)
{
lcd_gotoxy(15,0);	// Affichage flecheHaut
		         lcd_putchar(Caractere_FlecheH);	
			 lcd_gotoxy(15,1);	// Affichage flecheBas
			 lcd_putchar(Caractere_FlecheB);
}

// -------------------------------------------------------------------

int Led_BP_TOR(char entree)
{
  unsigned char BP, Affich=1, sortir=0;
  
  if (entree==0)
       {
        sprintf(display_buffer_ligne1,"Sortir = 2xECHAP");
    	Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1); 
       }
        else 
       {
        sprintf(display_buffer_ligne1,"Sortir = 2xPA2");
       	Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1); 
       }
  
  while(sortir<2)
  {
   BP=Lire_BP();
   switch (BP)
      {
       case INC: PORTB = 0x1F;
       		 sprintf(display_buffer_ligne0,"   PA5 -> ALL");
       		 Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
                 Affich = 1;
                 if (sortir=1) sortir=0;
       break; 
       
       case DEC: PORTB.4 = 1;
       		 sprintf(display_buffer_ligne0,"   PA4 -> PB4");
       	       	 Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1); 
                 Affich = 1;
                 if (sortir=1) sortir=0;
       	break;
       	 
     	case OK: PORTB.3 = 1;
     		 sprintf(display_buffer_ligne0,"   PA3 -> PB3");
       	       	 Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1); 
                 Affich = 1; 
                 if (sortir=1) sortir=0;
       	break;
       	  
      	case SET:PORTB.1 = 1;
      		 sprintf(display_buffer_ligne0,"   PA1 -> PB1");
        	 Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1); 
                 Affich = 1;
                 if (sortir=1) sortir=0;
       	break;
       	   
      	case ECHAP: PORTB.2 = 1;
      	            sprintf(display_buffer_ligne0,"   PA2 -> PB2");
       	       	    Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1); 
       		    Affich = 1;
       		    sortir = sortir + 1;
       	break;
       	
       	case ENTR: PORTB.0 = 1;
       		    sprintf(display_buffer_ligne0,"   PA0 -> PB0");
       	       	    Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1); 
                    Affich = 1;
                    if (sortir=1) sortir=0;
       	break;
       	     
      	case 0xff: PORTB = 0;
      	           if ((Affich == 1)&&(entree==0))
       	  	    {
       	  	      sprintf(display_buffer_ligne0," Test BPx->DELx");
       	       	      Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1); 
       	       	    }
       	       	    else if  ((Affich == 1)&&(entree==1))
       	       	    {
       	  	      sprintf(display_buffer_ligne0,"Test TORx->DELx");
       	       	      Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1); 
       	       	    }
       	       	    Affich = 0; 
       	       	    
       	break;
       }
  }
PORTB = 0;   
return 0;
}

